@@ -30,6 +30,10 @@ static gboolean gail_select_watcher     (GSignalInvocationHint *ihint,
                                          guint                  n_param_values,
                                          const GValue          *param_values,
                                          gpointer               data);
+static gboolean gail_set_focus_watcher  (GSignalInvocationHint *ihint,
+                                         guint                  n_param_values,
+                                         const GValue          *param_values,
+                                         gpointer               data);
 static void     gail_focus_tracker_init ();
 
 static GtkWidget* focus_widget = NULL;
@@ -41,7 +45,7 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
                     gpointer               data)
 {
   GObject *object;
-  AtkObject *aobject;
+  AtkObject *aobject = NULL;
   GtkWidget *widget;
   GdkEvent *event;
 
@@ -52,57 +56,107 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
   widget = GTK_WIDGET (object);
 
   if (event->type == GDK_FOCUS_CHANGE) 
-  {
-    if (event->focus_change.in)
     {
-      if (GTK_IS_WINDOW (widget) && GTK_WINDOW (widget)->focus_widget)
-      {
-        /*
-         * Found focus widget so use that
-         */
-        widget = GTK_WINDOW (widget)->focus_widget;
-      }
+      if (event->focus_change.in)
+        {
+          if (GTK_IS_WINDOW (widget))
+            {
+              GtkWindow *window;
+
+              window = GTK_WINDOW (widget);
+              if (window->focus_widget)
+                {
+                  /*
+                   * Found focus widget so use that unless it is already the
+                   * focus widget
+                   */
+                  if (focus_widget == window->focus_widget)
+                    {
+                      return TRUE;
+                    }
+                  else
+                    {
+                      widget = window->focus_widget;
+                    }
+                }
+              else if (window->type == GTK_WINDOW_POPUP)
+                {
+                  GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
+
+                  if (GTK_IS_WIDGET (child) && GTK_WIDGET_HAS_GRAB (child))
+                    {
+                      if (GTK_IS_MENU_SHELL (child))
+                        {
+                          if (GTK_MENU_SHELL (child)->active_menu_item)
+                            {
+                              /*
+                               * We have a menu which has a menu item selected
+                               * so we do not report focus on the menu.
+                               */ 
+                              return TRUE; 
+                            }
+                        }
+                      widget = child;
+                    } 
+                }
+            }
+        }
+      else
+        {
+          /* Ignore focus out */
+          return TRUE;
+        }
     }
-    else
-    {
-      /* Ignore focus out */
-      return TRUE;
-    }
-  }
   else
-  {
-    if (event->type == GDK_MOTION_NOTIFY && GTK_WIDGET_HAS_FOCUS (widget))
     {
-      if (widget == focus_widget)
-        return TRUE;
+      if (event->type == GDK_MOTION_NOTIFY && GTK_WIDGET_HAS_FOCUS (widget))
+        {
+          if (widget == focus_widget)
+            {
+              return TRUE;
+            }
+        }
+      else
+        {
+          return TRUE;
+        }
     }
-    else
-    {
-      return TRUE;
-    }
-  }
   focus_widget = widget;
 
   if (GTK_IS_ENTRY (widget))
-  {
-    GtkWidget *other_widget = widget->parent;
-    if (GTK_IS_COMBO (other_widget))
-      widget = other_widget;
-  } 
-
-  aobject = gtk_widget_get_accessible (widget);
-
-  if (GTK_IS_NOTEBOOK (widget)) 
-  {
-    gint page_num;
-
-    page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
-    if (page_num != -1)
     {
-      aobject = atk_object_ref_accessible_child (aobject, page_num);
-      g_object_unref (aobject);
+      GtkWidget *other_widget = widget->parent;
+      if (GTK_IS_COMBO (other_widget))
+        widget = other_widget;
+    } 
+  else if (GTK_IS_NOTEBOOK (widget)) 
+    {
+      gint page_num;
+
+      page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
+      if (page_num != -1)
+        {
+          aobject = gtk_widget_get_accessible (widget);
+          aobject = atk_object_ref_accessible_child (aobject, page_num);
+          g_object_unref (aobject);
+        }
     }
-  }
+  else if (GNOME_IS_CANVAS (widget)) 
+    {
+      GnomeCanvas *canvas;
+
+      canvas = GNOME_CANVAS (widget);
+
+      if (canvas->focused_item)
+        {
+          aobject = atk_gobject_accessible_for_object (G_OBJECT (canvas->focused_item));
+        }
+    }
+  if (aobject == NULL)
+    {
+      aobject = gtk_widget_get_accessible (widget);
+    }
+
   atk_focus_tracker_notify (aobject);
 
   return TRUE; 
@@ -128,9 +182,45 @@ gail_select_watcher (GSignalInvocationHint *ihint,
    * select signal on non-mapped widget
    * so ignore
    */
-  {
     return TRUE;
-  }
+
+  if (focus_widget == widget)
+  /*
+   * We may already have got a notification because of bug 68483
+   */
+    return TRUE;
+
+  focus_widget = widget;
+  aobject = gtk_widget_get_accessible (widget);
+
+  atk_focus_tracker_notify (aobject);
+
+  return TRUE; 
+}
+
+static gboolean
+gail_set_focus_watcher (GSignalInvocationHint *ihint,
+                        guint                  n_param_values,
+                        const GValue          *param_values,
+                        gpointer               data)
+{
+  GObject *object, *focus_object;
+  AtkObject *aobject;
+  GtkWidget *widget;
+
+  object = g_value_get_object (param_values + 0);
+  g_return_val_if_fail (GTK_IS_WIDGET(object), FALSE);
+
+  focus_object = g_value_get_object (param_values + 1);
+  if (focus_object)
+    {
+      g_return_val_if_fail (GTK_IS_WIDGET(focus_object), FALSE);
+      widget = GTK_WIDGET (focus_object);
+    }
+  else
+    {
+      widget = GTK_WIDGET (object);
+    }
 
   focus_widget = widget;
   aobject = gtk_widget_get_accessible (widget);
@@ -146,42 +236,37 @@ gail_focus_tracker_init ()
   static gboolean  emission_hooks_added = FALSE;
 
   if (!emission_hooks_added)
-  {
-    /*
-     * We cannot be sure that the classes exist so we make sure that they do.
-     */
-    gtk_type_class (GTK_TYPE_WIDGET);
-    gtk_type_class (GTK_TYPE_ITEM);
+    {
+      /*
+       * We cannot be sure that the classes exist so we make sure that they do.
+       */
+      gtk_type_class (GTK_TYPE_WIDGET);
+      gtk_type_class (GTK_TYPE_ITEM);
 
-    /*
-     * We listen for event_after signal and then check that the
-     * event was a focus in event so we get called after the event.
-     */
-    g_signal_add_emission_hook (
+      /*
+       * We listen for event_after signal and then check that the
+       * event was a focus in event so we get called after the event.
+       */
+      g_signal_add_emission_hook (
              g_signal_lookup ("event-after", GTK_TYPE_WIDGET), 0,
              gail_focus_watcher, NULL, (GDestroyNotify) NULL);
-    /*
-     * A "select" signal is emitted when arrow key is used to
-     * move to a list item in the popup window of a GtkCombo or
-     * a menu item in a menu.
-     */
-    g_signal_add_emission_hook (
+      /*
+       * A "select" signal is emitted when arrow key is used to
+       * move to a list item in the popup window of a GtkCombo or
+       * a menu item in a menu.
+       */
+      g_signal_add_emission_hook (
              g_signal_lookup ("select", GTK_TYPE_ITEM), 0,
              gail_select_watcher, NULL, (GDestroyNotify) NULL);
-    /*
-     * Activating a menu item means that it is first selected
-     * so we omit these.
-     */
-#if 0
-    signal_id = g_signal_lookup ("activate_item", GTK_TYPE_MENU_ITEM);
-    g_signal_add_emission_hook (signal_id, 0,
-                                gail_event_watcher, NULL, (GDestroyNotify) NULL);
-    signal_id = g_signal_lookup ("activate", GTK_TYPE_MENU_ITEM);
-    g_signal_add_emission_hook (signal_id, 0,
-                                gail_event_watcher, NULL, (GDestroyNotify) NULL);
-#endif
-    emission_hooks_added = TRUE;
-  }
+      /*
+       * A "set-focus" signal on a GtkWindow may set the focus
+       */
+      g_signal_add_emission_hook (
+             g_signal_lookup ("set-focus", GTK_TYPE_WINDOW), 0,
+             gail_set_focus_watcher, NULL, (GDestroyNotify) NULL);
+
+      emission_hooks_added = TRUE;
+    }
 }
 
 int
@@ -279,8 +364,10 @@ gtk_module_init(gint *argc, char** argv[])
 				 GAIL_TYPE_CANVAS_TEXT_FACTORY);
   atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_WIDGET,
 				 GAIL_TYPE_CANVAS_WIDGET_FACTORY);
+  atk_registry_set_factory_type (default_registry, GNOME_TYPE_CANVAS_ITEM,
+				 GAIL_TYPE_CANVAS_ITEM_FACTORY);
 
-    atk_focus_tracker_init (gail_focus_tracker_init);
+  atk_focus_tracker_init (gail_focus_tracker_init);
 
   /* Initialize the GailUtility class */
   data = g_type_class_ref (GAIL_TYPE_UTIL);
