@@ -106,8 +106,8 @@ typedef struct _GailCListCellData   GailCListCellData;
 
 
 static void       gail_clist_class_init            (GailCListClass    *klass);
-static void       gail_clist_real_init             (GailWidget        *widget,
-                                                    GtkWidget         *gtk_widget);
+static void       gail_clist_real_initialize       (AtkObject         *obj,
+                                                    gpointer          data);
 static void       gail_clist_finalize              (GObject           *object);
 
 static gint       gail_clist_get_n_children        (AtkObject         *obj);
@@ -314,16 +314,12 @@ gail_clist_class_init (GailCListClass *klass)
 {
   AtkObjectClass *class = ATK_OBJECT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GailWidgetClass *widget_class;
 
-  widget_class = (GailWidgetClass*)klass;
-
-  parent_class = g_type_class_ref (GAIL_TYPE_CONTAINER);
+  parent_class = g_type_class_peek_parent (klass);
  
   class->get_n_children = gail_clist_get_n_children;
   class->ref_child = gail_clist_ref_child;
-
-  widget_class->init = gail_clist_real_init;
+  class->initialize = gail_clist_real_initialize;
 
   gobject_class->finalize = gail_clist_finalize;
 }
@@ -338,32 +334,32 @@ gail_clist_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_CLIST, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   accessible->role = ATK_ROLE_TABLE;
 
   return accessible;
 }
 
 static void
-gail_clist_real_init (GailWidget         *widget,
-                      GtkWidget          *gtk_widget)
+gail_clist_real_initialize (AtkObject *obj,
+                            gpointer  data)
 {
   GailCList *clist;
   GtkCList *gtk_clist;
   gint i;
 
-  GAIL_WIDGET_CLASS (parent_class)->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  clist = GAIL_CLIST (widget);
+  clist = GAIL_CLIST (obj);
 
   clist->caption = NULL;
   clist->summary = NULL;
   clist->row_data = NULL;
   clist->cell_data = NULL;
 
-  gtk_clist = GTK_CLIST (gtk_widget);
+  gtk_clist = GTK_CLIST (data);
  
   clist->n_cols = gtk_clist->columns;
   clist->columns = g_new (GailCListColumn, gtk_clist->columns);
@@ -378,22 +374,28 @@ gail_clist_real_init (GailWidget         *widget,
   g_signal_connect (gtk_clist,
                     "select-row",
                     G_CALLBACK (gail_clist_select_row_gtk),
-                    widget);
+                    obj);
   g_signal_connect (gtk_clist,
                     "unselect-row",
                     G_CALLBACK (gail_clist_unselect_row_gtk),
-                    widget);
+                    obj);
   /*
    * Adjustment callbacks
    */
-  g_signal_connect (gtk_clist->hadjustment,
-                    "value_changed",
-                    G_CALLBACK (gail_clist_adjustment_changed),
-                    gtk_clist);
-  g_signal_connect (gtk_clist->vadjustment,
-                    "value_changed",
-                    G_CALLBACK (gail_clist_adjustment_changed),
-                    gtk_clist);
+  if (gtk_clist->hadjustment)
+    {
+      g_signal_connect (gtk_clist->hadjustment,
+                        "value_changed",
+                        G_CALLBACK (gail_clist_adjustment_changed),
+                        gtk_clist);
+    }
+  if (gtk_clist->vadjustment)
+    {
+      g_signal_connect (gtk_clist->vadjustment,
+                        "value_changed",
+                        G_CALLBACK (gail_clist_adjustment_changed),
+                        gtk_clist);
+    }
 }
 
 static void
@@ -812,6 +814,9 @@ gail_clist_get_column_header (AtkTable      *table,
   if (return_widget == NULL)
     return NULL;
 
+  g_return_if_fail (GTK_IS_BIN (return_widget));
+  return_widget = gtk_bin_get_child (GTK_BIN(return_widget));
+
   return gtk_widget_get_accessible (return_widget);
 }
 
@@ -1338,7 +1343,7 @@ gail_clist_set_row_data (AtkTable      *table,
           break;
         }
     } 
-  if (found == FALSE)
+  if (!found)
     {
       row_data = g_new (GailCListRow, 1);
       row_data->row_number = row;
