@@ -22,15 +22,15 @@
 #include "gailentry.h"
 #include "gailentryfactory.h"
 #include "gailcombo.h"
-#include "gailtexthelper.h"
+#include <util/gailmisc.h>
 
 static void       gail_entry_class_init            (GailEntryClass       *klass);
 static void       gail_entry_object_init           (GailEntry            *entry);
-static void	  gail_entry_real_init             (GailWidget           *widget,
-                                                    GtkWidget            *gtk_widget);
-static void	  gail_entry_real_notify_gtk	   (GObject		*obj,
-                                                    GParamSpec		*pspec);
-static void       gail_entry_finalize              (GObject            *object);
+static void	  gail_entry_real_initialize       (AtkObject            *obj,
+                                                    gpointer             data);
+static void	  gail_entry_real_notify_gtk	   (GObject		 *obj,
+                                                    GParamSpec		 *pspec);
+static void       gail_entry_finalize              (GObject              *object);
 
 static gint       gail_entry_get_index_in_parent   (AtkObject            *accessible);
 
@@ -202,47 +202,49 @@ gail_entry_class_init (GailEntryClass *klass)
 
   widget_class = (GailWidgetClass*)klass;
 
-  parent_class = g_type_class_ref (GAIL_TYPE_WIDGET);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gail_entry_finalize;
 
   class->ref_state_set = gail_entry_ref_state_set;
   class->get_index_in_parent = gail_entry_get_index_in_parent;
+  class->initialize = gail_entry_real_initialize;
 
-  widget_class->init = gail_entry_real_init;
   widget_class->notify_gtk = gail_entry_real_notify_gtk;
 }
 
 static void
 gail_entry_object_init (GailEntry *entry)
 {
-  entry->texthelper = NULL;
+  entry->textutil = NULL;
   entry->signal_name = NULL;
   entry->cursor_position = 0;
   entry->selection_bound = 0;
 }
 
 static void
-gail_entry_real_init (GailWidget *widget, 
-                      GtkWidget  *gtk_widget)
+gail_entry_real_initialize (AtkObject *obj, 
+                            gpointer  data)
 {
   GtkEntry *entry;
+  GailEntry *gail_entry;
   const gchar *entry_text;
 
-  parent_class->init (widget, gtk_widget);
+  ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
-  GAIL_ENTRY (widget)->texthelper = gail_text_helper_new ();
+  gail_entry = GAIL_ENTRY (obj);
+  gail_entry->textutil = gail_text_util_new ();
   
-  entry = GTK_ENTRY (gtk_widget);
+  entry = GTK_ENTRY (data);
   entry_text = gtk_entry_get_text (entry);
-  gail_text_helper_text_setup (GAIL_ENTRY(widget)->texthelper, entry_text);
+  gail_text_util_text_setup (gail_entry->textutil, entry_text);
 
   /* Set up signal callbacks */
-  g_signal_connect (gtk_widget, "insert-text",
+  g_signal_connect (data, "insert-text",
 	G_CALLBACK (_gail_entry_insert_text_cb), NULL);
-  g_signal_connect (gtk_widget, "delete-text",
+  g_signal_connect (data, "delete-text",
 	G_CALLBACK (_gail_entry_delete_text_cb), NULL);
-  g_signal_connect (gtk_widget, "changed",
+  g_signal_connect (data, "changed",
 	G_CALLBACK (_gail_entry_changed_cb), NULL);
 }
 
@@ -288,7 +290,7 @@ gail_entry_finalize (GObject            *object)
 {
   GailEntry *entry = GAIL_ENTRY (object);
 
-  g_object_unref (entry->texthelper);
+  g_object_unref (entry->textutil);
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -302,9 +304,9 @@ gail_entry_new (GtkWidget *widget)
 
   object = g_object_new (GAIL_TYPE_ENTRY, NULL);
 
-  gail_widget_init (GAIL_WIDGET (object), widget);
-
   accessible = ATK_OBJECT (object);
+  atk_object_initialize (accessible, widget);
+
   if (GTK_ENTRY (widget)->visible)
     accessible->role = ATK_ROLE_TEXT;
   else
@@ -416,7 +418,7 @@ gail_entry_get_text_before_offset (AtkText	    *text,
   /* Get Entry */
   entry = GTK_ENTRY (widget);
 
-  return gail_text_helper_get_text (GAIL_ENTRY (text)->texthelper,
+  return gail_text_util_get_text (GAIL_ENTRY (text)->textutil,
                           gtk_entry_get_layout (entry), GAIL_BEFORE_OFFSET, 
                           boundary_type, offset, start_offset, end_offset);
 }
@@ -439,7 +441,7 @@ gail_entry_get_text_at_offset (AtkText          *text,
   /* Get Entry */
   entry = GTK_ENTRY (widget);
 
-  return gail_text_helper_get_text (GAIL_ENTRY (text)->texthelper,
+  return gail_text_util_get_text (GAIL_ENTRY (text)->textutil,
                             gtk_entry_get_layout (entry), GAIL_AT_OFFSET, 
                             boundary_type, offset, start_offset, end_offset);
 }
@@ -462,7 +464,7 @@ gail_entry_get_text_after_offset  (AtkText	    *text,
   /* Get Entry */
   entry = GTK_ENTRY (widget);
 
-  return gail_text_helper_get_text (GAIL_ENTRY (text)->texthelper,
+  return gail_text_util_get_text (GAIL_ENTRY (text)->textutil,
                            gtk_entry_get_layout (entry), GAIL_AFTER_OFFSET, 
                            boundary_type, offset, start_offset, end_offset);
 }
@@ -536,12 +538,12 @@ gail_entry_get_run_attributes (AtkText *text,
   dir = gtk_widget_get_direction (widget);
   if (dir == GTK_TEXT_DIR_RTL)
     {
-      at_set = gail_text_helper_add_attribute (at_set,
-                                               ATK_TEXT_ATTR_DIRECTION,
-       g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_DIRECTION, dir)));
+      at_set = gail_misc_add_attribute (at_set,
+                                        ATK_TEXT_ATTR_DIRECTION,
+       g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_DIRECTION, dir)));
     }
 
-  at_set = gail_text_helper_get_run_attributes (at_set,
+  at_set = gail_misc_layout_get_run_attributes (at_set,
                                                 gtk_entry_get_layout (entry),
                                                 entry->text,
                                                 offset,
@@ -564,16 +566,15 @@ gail_entry_get_default_attributes (AtkText *text)
 
   entry = GTK_ENTRY (widget);
 
-  at_set = gail_text_helper_add_attribute (at_set,
-                                           ATK_TEXT_ATTR_JUSTIFICATION,
-     g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_JUSTIFICATION, 0)));
-  at_set = gail_text_helper_add_attribute (at_set,
-                                           ATK_TEXT_ATTR_WRAP_MODE,
-              g_strdup (atk_attribute_get_value (ATK_TEXT_ATTR_WRAP_MODE, 0)));
-  at_set = gail_text_helper_get_default_attributes (at_set,
-                                                    gtk_entry_get_layout (entry)
-,
-                                                    widget);
+  at_set = gail_misc_add_attribute (at_set,
+                                    ATK_TEXT_ATTR_JUSTIFICATION,
+     g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_JUSTIFICATION, 0)));
+  at_set = gail_misc_add_attribute (at_set,
+                                    ATK_TEXT_ATTR_WRAP_MODE,
+          g_strdup (atk_text_attribute_get_value (ATK_TEXT_ATTR_WRAP_MODE, 0)));
+  at_set = gail_misc_get_default_attributes (at_set,
+                                             gtk_entry_get_layout (entry),
+                                             widget);
   return at_set;
 }
   
@@ -606,7 +607,7 @@ gail_entry_get_character_extents (AtkText *text,
     index += entry->preedit_length;
   pango_layout_index_to_pos (gtk_entry_get_layout(entry), index, &char_rect);
  
-  gail_text_helper_get_extents_from_pango_rectangle (widget, &char_rect, 
+  gail_misc_get_extents_from_pango_rectangle (widget, &char_rect, 
                         x_layout, y_layout, x, y, width, height, coords);
 } 
 
@@ -629,7 +630,7 @@ gail_entry_get_offset_at_point (AtkText *text,
   
   gtk_entry_get_layout_offsets (entry, &x_layout, &y_layout);
   
-  index = gail_text_helper_get_index_at_point_in_layout (widget, 
+  index = gail_misc_get_index_at_point_in_layout (widget, 
                gtk_entry_get_layout(entry), x_layout, y_layout, x, y, coords);
   if (index == -1)
     return -1;  
@@ -1027,8 +1028,7 @@ _gail_entry_changed_cb (GtkEntry *entry)
   /*
    * Update the text in the GtkTextBuffer
    */
-  gail_text_helper_text_setup (gail_entry->texthelper, 
-                               gtk_entry_get_text(entry));
+  gail_text_util_text_setup (gail_entry->textutil, gtk_entry_get_text(entry));
 }
 
 static gboolean 
