@@ -28,7 +28,7 @@ static void gail_frame_factory_class_init (
 static AtkObject* gail_frame_factory_create_accessible (
                                 GObject                       *obj);
 
-static AtkObjectFactoryClass    *parent_class = NULL;
+static GType gail_frame_factory_get_accessible_type (void);
 
 GType
 gail_frame_factory_get_type ()
@@ -62,9 +62,8 @@ gail_frame_factory_class_init (GailFrameFactoryClass *klass)
 {
   AtkObjectFactoryClass *class = ATK_OBJECT_FACTORY_CLASS (klass);
 
-  parent_class = g_type_class_ref (ATK_TYPE_OBJECT_FACTORY);
-
   class->create_accessible = gail_frame_factory_create_accessible;
+  class->get_accessible_type = gail_frame_factory_get_accessible_type;
 }
 
 AtkObjectFactory* gail_frame_factory_new ()
@@ -87,3 +86,9 @@ gail_frame_factory_create_accessible (GObject     *obj)
 
   return gail_frame_new (widget);
 }
+
+static GType
+gail_frame_factory_get_accessible_type (void)
+{
+  return GAIL_TYPE_FRAME;
+}
