@@ -27,7 +27,7 @@ static void gail_image_factory_class_init (GailImageFactoryClass        *klass);
 static AtkObject* gail_image_factory_create_accessible (
                                 GObject                                 *obj);
 
-static AtkObjectFactoryClass    *parent_class = NULL;
+static GType gail_image_factory_get_accessible_type (void);
 
 GType
 gail_image_factory_get_type ()
@@ -61,12 +61,12 @@ gail_image_factory_class_init (GailImageFactoryClass *klass)
 {
   AtkObjectFactoryClass *class = ATK_OBJECT_FACTORY_CLASS (klass);
 
-  parent_class = g_type_class_ref (ATK_TYPE_OBJECT_FACTORY);
-
   class->create_accessible = gail_image_factory_create_accessible;
+  class->get_accessible_type = gail_image_factory_get_accessible_type;
 }
 
-AtkObjectFactory* gail_image_factory_new ()
+AtkObjectFactory*
+gail_image_factory_new ()
 {
   GObject *factory;
 
@@ -85,3 +85,9 @@ gail_image_factory_create_accessible (GObject     *obj)
   widget = GTK_WIDGET (obj);
   return gail_image_new (widget);
 }
+
+static GType
+gail_image_factory_get_accessible_type (void)
+{
+  return GAIL_TYPE_IMAGE;
+}
