@@ -22,7 +22,7 @@
 
 #include <gtk/gtkaccessible.h>
 #include <gail/gailcontainer.h>
-#include <gail/gailtexthelper.h>
+#include <util/gailtextutil.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -42,7 +42,7 @@ struct _GailTextView
 {
   GailContainer  parent;
 
-  GailTextHelper *texthelper;
+  GailTextUtil   *textutil;
   gint           previous_insert_offset;
   /*
    * These fields store information about text changed
