@@ -27,8 +27,7 @@ static void gail_widget_factory_class_init (
 
 static AtkObject* gail_widget_factory_create_accessible (
                                 GObject                     *obj);
-
-static GailWidgetFactoryClass    *parent_class = NULL;
+static GType gail_widget_factory_get_accessible_type (void);
 
 GType
 gail_widget_factory_get_type ()
@@ -63,9 +62,8 @@ gail_widget_factory_class_init (GailWidgetFactoryClass *klass)
 {
   AtkObjectFactoryClass *class = ATK_OBJECT_FACTORY_CLASS (klass);
 
-  parent_class = g_type_class_ref (ATK_TYPE_OBJECT_FACTORY);
-
   class->create_accessible = gail_widget_factory_create_accessible;
+  class->get_accessible_type = gail_widget_factory_get_accessible_type;
 }
 
 AtkObjectFactory* 
@@ -88,3 +86,9 @@ gail_widget_factory_create_accessible (GObject   *obj)
   widget = GTK_WIDGET (obj);
   return gail_widget_new (widget);
 }
+
+static GType
+gail_widget_factory_get_accessible_type (void)
+{
+  return GAIL_TYPE_WIDGET;
+}
