@@ -4,12 +4,12 @@
 #define MAX_NAME_VALUE 20
 
 #include <netinet/in.h>
-#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
 #include <errno.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <strings.h>
 #include "testlib.h"
 
@@ -24,7 +24,7 @@ typedef enum
   TEXT,
   VALUE,
   END_TABS
-}TabNumber;
+} TabNumber;
 
 typedef enum
 {
@@ -39,7 +39,7 @@ typedef enum
   TEXT_INTERFACE,
   TEXT_ATTRIBUTES,
   VALUE_INTERFACE
-}GroupId;
+} GroupId;
 
 typedef enum
 {
@@ -47,7 +47,7 @@ typedef enum
   VALUE_BOOLEAN,
   VALUE_TEXT,
   VALUE_BUTTON
-}ValueType;
+} ValueType;
 
 /* GUI Information for the group */
 
@@ -62,7 +62,7 @@ typedef struct
   gchar         *name;
   gboolean      is_scrolled;
   gint          default_height;
-}GroupInfo;
+} GroupInfo;
 
 typedef struct
 {
@@ -70,7 +70,7 @@ typedef struct
   GtkWidget *page;
   GtkWidget *main_box;
   gchar     *name;
-}TabInfo;
+} TabInfo;
 
 typedef struct
 {
@@ -89,7 +89,7 @@ typedef struct
   GtkWidget *string;
   GtkWidget *boolean;
   GtkWidget *text;
-}NameValue;
+} NameValue;
 
 typedef enum {
    FERRET_SIGNAL_OBJECT,
@@ -165,9 +165,9 @@ static NameValue* _get_name_value(GroupInfo *group, const gchar *label,
 
 static void _update_handlers(AtkObject *obj);
 static void _notify_text_insert_handler (GObject *obj,
-  int start_offset, int end_offset);
+  int position, int offset);
 static void _notify_text_delete_handler (GObject *obj,
-  int start_offset, int end_offset);
+  int position, int offset);
 static void _notify_caret_handler (GObject *obj, int position);
 static void _notify_table_row_inserted (GObject *obj,
   gint start_offset, gint length);
@@ -183,6 +183,8 @@ static void _notify_object_child_added (GObject *obj,
   gint index, AtkObject *child);
 static void _notify_object_child_removed (GObject *obj,
   gint index, AtkObject *child);
+static void _notify_object_state_change (GObject *obj,
+  gchar *name, gboolean set);
 
 /* Property handlers */
 
@@ -204,15 +206,19 @@ void _toggle_festival_terse(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data);
 void _toggle_trackmouse(GtkCheckMenuItem *checkmenuitem,
   gpointer user_data);
+void _toggle_trackfocus(GtkCheckMenuItem *checkmenuitem,
+  gpointer user_data);
 
 /* Global variables */
 static GtkNotebook *notebook;
 static TabInfo  *nbook_tabs[END_TABS];
-static gboolean mouse_watcher_focus_id = -1;
-static gboolean mouse_watcher_button_id = -1;
+static gint mouse_watcher_focus_id = -1;
+static gint mouse_watcher_button_id = -1;
+static gint focus_tracker_id = -1;
 static gboolean use_magnifier = FALSE;
 static gboolean use_festival = FALSE;
 static gboolean track_mouse = FALSE;
+static gboolean track_focus = TRUE;
 static gboolean say_role = TRUE;
 static gboolean say_accel = TRUE;
 static gboolean display_ascii = FALSE;
@@ -231,6 +237,7 @@ static GtkWidget *menuitem_magnifier = NULL;
 static GtkWidget *menuitem_festival = NULL;
 static GtkWidget *menuitem_festival_terse = NULL;
 static GtkWidget *menuitem_trackmouse = NULL;
+static GtkWidget *menuitem_trackfocus = NULL;
 
 static struct sockaddr_un mag_server = { AF_UNIX , "/tmp/magnifier_socket" };
 static struct sockaddr_un client = { AF_UNIX, "/tmp/mag_client"};
@@ -309,6 +316,8 @@ static void _festival_say (const gchar *text)
   gchar *quoted;
   gchar *p;
 
+  fprintf (stderr, "saying %s\n", text);
+
   if (!fd)
     {
       fd = _festival_init ();
@@ -337,17 +346,17 @@ static void _send_to_festival (const gchar *role_name,
   const gchar *name, const gchar *accel)
 {
   gchar *string;
-  int len = (strlen (role_name)+1 - 9 + strlen (name)+2 + 4 + strlen (accel)+2);
-                   /* omit " atk_role_" and add 2 additional spaces */
+  int len = (strlen (role_name)+1 + strlen (name)+2 + 4 + strlen (accel)+2);
   int i, j;
   gchar ch;
   gchar *accel_name;
+  
   string = (gchar *) g_malloc (len * sizeof (gchar));
 
   i = 0;
   if (say_role)
     {
-      j = 9;
+      j = 0;
       while (role_name[j])
         {
           ch = role_name[j++];
@@ -389,6 +398,7 @@ static void _send_to_festival (const gchar *role_name,
         };
     }
   string[i] = '\0';
+
   _festival_say (string);
   g_free (string);
 }
@@ -471,7 +481,7 @@ static void _print_accessible (AtkObject *aobject)
     g_print("\nFocus change\n");
 
   /* Do not attach signal handlers if the user has asked not to */
-  if (no_signals == FALSE)
+  if (!no_signals)
     _update_handlers(aobject);
   else
     last_object = aobject; /* _update_handler normally does this */
@@ -479,14 +489,45 @@ static void _print_accessible (AtkObject *aobject)
   top_tab = gtk_notebook_get_current_page (notebook) + OBJECT;
   _update(top_tab, aobject);
 
-  if (use_magnifier && (ATK_IS_COMPONENT (aobject)))
+  if (use_magnifier)
     {
       gint x, y;
-      gint w, h;
-      atk_component_get_extents (ATK_COMPONENT(aobject),
-                                 &x, &y, &w, &h,
-                                 ATK_XY_SCREEN);
-      _send_to_magnifier (x, y, w, h);
+      gint w=-1, h;
+      
+      if (ATK_IS_TEXT (aobject))
+        {
+	  gint x0, y0, w0, h0;
+	  gint xN, yN, wN, hN;
+	  gint len;
+	  len = atk_text_get_character_count (ATK_TEXT (aobject));
+	  atk_text_get_character_extents (ATK_TEXT (aobject), 0,
+					  &x0, &y0, &w0, &h0,
+					  ATK_XY_SCREEN);
+          atk_text_get_character_extents (ATK_TEXT (aobject), len-1,
+					  &xN, &yN, &wN, &hN,
+					  ATK_XY_SCREEN);
+	  x = MIN (x0, xN);
+	  y = MIN (y0, yN);
+	  /*
+	   * due to text coord bugs in gail for GailTextCell
+	   *  we kludge this
+	   */
+	  if (ATK_IS_COMPONENT (aobject))
+	    {
+	      atk_component_get_extents (ATK_COMPONENT (aobject),
+					 &x, &y, &w, &h,
+					 ATK_XY_SCREEN);
+	    }
+	  w = MAX (x0+w0, xN+wN) - x;
+	  h = MAX (y0+h0, yN+hN) - y;
+        } 
+      else if (ATK_IS_COMPONENT (aobject))
+        {
+	  atk_component_get_extents (ATK_COMPONENT(aobject),
+				     &x, &y, &w, &h,
+				     ATK_XY_SCREEN);
+        }
+      if (w > -1) _send_to_magnifier (x, y, w, h);
     }
 }
 
@@ -529,6 +570,7 @@ _print_object (AtkObject *aobject)
     G_CONST_RETURN gchar * parent_typename = NULL;
     G_CONST_RETURN gchar * role_name = NULL;
     G_CONST_RETURN gchar * accel_name = NULL;
+    G_CONST_RETURN gchar * text = NULL;
     AtkRole role;
     AtkObject *parent = NULL;
     static AtkObject *prev_aobject = NULL;
@@ -548,7 +590,7 @@ _print_object (AtkObject *aobject)
       index_in_parent = atk_object_get_index_in_parent(aobject);
     n_children = atk_object_get_n_accessible_children(aobject);
     role = atk_object_get_role(aobject);
-    role_name = get_role_name(role);
+    role_name = atk_role_get_name(role);
 
     if (ATK_IS_ACTION (aobject))
       {
@@ -598,17 +640,25 @@ _print_object (AtkObject *aobject)
       {
         if (aobject != prev_aobject)
           {
-            if (!name) name = "no name";
-            _send_to_festival (role_name, name, accel_name);
-	    if (ATK_IS_TEXT (aobject))
-	    {
-		    fprintf (stderr, "first sentence: %s\n",
-			     atk_text_get_text_at_offset (ATK_TEXT (aobject),
-							  (gint) 0,
-							  ATK_TEXT_BOUNDARY_SENTENCE_END,
-							  (gint *) NULL,
-							  (gint *) NULL));
-	    }
+	    if (ATK_IS_TEXT (aobject) && !name)
+	      {
+		text = 
+		  atk_text_get_text_at_offset (ATK_TEXT (aobject),
+					       (gint) 0,
+					       ATK_TEXT_BOUNDARY_SENTENCE_END,
+					       (gint *) NULL,
+					       (gint *) NULL);
+		fprintf (stderr, "first sentence: %s\n", text);
+		_send_to_festival (role_name, 
+				   text, "");
+		if (!name) name = "no name";
+	      }
+            else 
+	      { 
+		text = "";
+		if (!name) name = "no name";
+		_send_to_festival (role_name, name, accel_name);
+	      }
           }
       }
 
@@ -1326,7 +1376,7 @@ _print_value_type(gint group_num, gchar *type, GValue *value)
 static void
 _create_event_watcher ()
 {
-    atk_add_focus_tracker (_print_accessible);
+    focus_tracker_id = atk_add_focus_tracker (_print_accessible);
 
     if (track_mouse)
       {
@@ -1408,9 +1458,15 @@ _button_watcher (GSignalInvocationHint *ihint,
               g_object_unref (child);
             }
           else
-               g_print ("No child at position %d %d\n", 
-                        x,
-                        y);
+            {
+              if (!GTK_IS_MENU_ITEM (widget))
+                {
+                  g_print ("No child at position %d %d for %s\n", 
+                           x,
+                           y,
+                           g_type_name (G_OBJECT_TYPE (widget)));
+                }
+            }
         }
     }
 
@@ -1564,12 +1620,14 @@ _create_window ()
 
         _add_menu(&menu, &menuitem_trackmouse, "Track Mouse", track_mouse,
            GTK_SIGNAL_FUNC(_toggle_trackmouse));
+        _add_menu(&menu, &menuitem_trackfocus, "Track Focus", track_focus,
+           GTK_SIGNAL_FUNC(_toggle_trackfocus));
         _add_menu(&menu, &menuitem_magnifier, "Magnifier", use_magnifier,
            GTK_SIGNAL_FUNC(_toggle_magnifier));
         _add_menu(&menu, &menuitem_festival, "Festival", use_festival,
            GTK_SIGNAL_FUNC(_toggle_festival));
         _add_menu(&menu, &menuitem_festival_terse, "Festival Terse",
-          (say_role == FALSE && say_accel == FALSE),
+          (!say_role && !say_accel),
           GTK_SIGNAL_FUNC(_toggle_festival_terse));
         _add_menu(&menu, &menuitem_terminal, "Terminal Output", display_ascii,
            GTK_SIGNAL_FUNC(_toggle_terminal));
@@ -1601,25 +1659,25 @@ _add_menu(GtkWidget ** menu, GtkWidget ** menuitem, gchar * name,
 int
 gtk_module_init(gint argc, char* argv[])
 {
-    if (getenv ("FERRET_ASCII"))
+    if (g_getenv ("FERRET_ASCII"))
        display_ascii = TRUE;
 
-    if (getenv ("FERRET_NOSIGNALS"))
+    if (g_getenv ("FERRET_NOSIGNALS"))
        no_signals = TRUE;
 
     if (display_ascii)
        g_print("GTK ferret Module loaded\n");
 
-    if (getenv("FERRET_MAGNIFIER"))
+    if (g_getenv("FERRET_MAGNIFIER"))
         use_magnifier = TRUE;
 
-    if (getenv ("FERRET_FESTIVAL"))
+    if (g_getenv ("FERRET_FESTIVAL"))
         use_festival = TRUE;
 
-    if (getenv ("FERRET_MOUSETRACK"))
+    if (g_getenv ("FERRET_MOUSETRACK"))
         track_mouse = TRUE;
 
-    if (getenv ("FERRET_TERSE"))
+    if (g_getenv ("FERRET_TERSE"))
       {
         say_role = FALSE;
         say_accel = FALSE;
@@ -1722,7 +1780,7 @@ _get_group(TabInfo *tab, GroupId group_id, const gchar *groupname)
           }
       }
 
-   if (found == FALSE)
+   if (!found)
      {
        /* build a new one */
        group = (GroupInfo *)g_new0(GroupInfo, 1);
@@ -1830,14 +1888,14 @@ _get_name_value(GroupInfo *group, const gchar *label,
     for (nv_list = group->name_value; nv_list; nv_list = nv_list->next)
       {
         name_value = (NameValue *) nv_list->data;
-        if (name_value->active == FALSE)
+        if (!name_value->active)
           {
             found = TRUE;
             break;
           }
       }
 
-    if (found == FALSE)
+    if (!found)
       {
         name_value = (NameValue *)g_new0(NameValue, 1);
         name_value->column1 = GTK_HBOX(gtk_hbox_new(FALSE, 10));
@@ -1962,7 +2020,7 @@ _print_key_value(TabNumber tab_n, gint group_number,
     {
       if (type == VALUE_BOOLEAN)
         {
-          if (*((gboolean *)value) == TRUE)
+          if (*((gboolean *)value))
               g_print("\t%-30s\tTRUE\n", label);
           else
               g_print("\t%-30s\tFALSE\n", label);
@@ -2001,6 +2059,7 @@ _finished_group(TabNumber tab_no, gint group_number)
 
 static gulong child_added_id = 0;
 static gulong child_removed_id = 0;
+static gulong state_change_id = 0;
 
 static gulong text_caret_handler_id = 0;
 static gulong text_inserted_id = 0;
@@ -2026,6 +2085,8 @@ _update_handlers(AtkObject *obj)
            g_signal_handler_disconnect(last_object, child_added_id);
         if (child_removed_id != 0)
            g_signal_handler_disconnect(last_object, child_removed_id);
+        if (state_change_id != 0)
+           g_signal_handler_disconnect(last_object, state_change_id);
 
         if (text_caret_handler_id != 0)
            g_signal_handler_disconnect(last_object, text_caret_handler_id);
@@ -2067,7 +2128,7 @@ _update_handlers(AtkObject *obj)
     table_column_reordered_id = 0;
     property_id = 0;
 
-    if (G_TYPE_CHECK_INSTANCE(obj) == FALSE)
+    if (!G_TYPE_CHECK_INSTANCE(obj))
         return;
 
     g_object_ref(obj);
@@ -2086,6 +2147,11 @@ _update_handlers(AtkObject *obj)
                 "children_changed::remove",
                 g_cclosure_new (G_CALLBACK (_notify_object_child_removed),
                 NULL, NULL), FALSE);
+
+         state_change_id = g_signal_connect_closure (obj,
+                "state_change",
+                g_cclosure_new (G_CALLBACK (_notify_object_state_change),
+                NULL, NULL), FALSE);
       }
 
     if (ATK_IS_TEXT(obj))
@@ -2142,20 +2208,22 @@ _update_handlers(AtkObject *obj)
 /* Text signals */
 
 static void
-_notify_text_insert_handler (GObject *obj, int start_offset, int end_offset)
+_notify_text_insert_handler (GObject *obj, int position, int offset)
 {
-    gchar *output_str = g_strdup_printf("position %d, length %d!",
-      start_offset, end_offset);
+    gchar *text = atk_text_get_text (ATK_TEXT (obj), position, position + offset);
+    gchar *output_str = g_strdup_printf("position %d, length %d text: %s",
+      position, offset,  text ? text: "<NULL>");
     _print_signal(ATK_OBJECT(obj), FERRET_SIGNAL_TEXT,
       "Text Insert", output_str);
     g_free(output_str);
 }
 
 static void
-_notify_text_delete_handler (GObject *obj, int start_offset, int end_offset)
+_notify_text_delete_handler (GObject *obj, int position, int offset)
 {
-    gchar *output_str = g_strdup_printf("position %d, length %d!",
-      start_offset, end_offset);
+    gchar *text = atk_text_get_text (ATK_TEXT (obj), position, position + offset);
+    gchar *output_str = g_strdup_printf("position %d, length %d text: %s",
+      position, offset,  text ? text: "<NULL>");
     _print_signal(ATK_OBJECT(obj), FERRET_SIGNAL_TEXT,
       "Text Delete", output_str);
     g_free(output_str);
@@ -2254,6 +2322,17 @@ _notify_object_child_removed (GObject *obj, gint index,
     g_free(output_str);
 }
 
+static void 
+_notify_object_state_change (GObject *obj, gchar *name, gboolean set)
+{
+    gchar *output_str = g_strdup_printf("name %s %s set", 
+                        name, set ? "is" : "was");
+    _print_signal(ATK_OBJECT(obj), FERRET_SIGNAL_OBJECT,
+      "State Change", output_str);
+    g_free(output_str);
+}
+
+
 /* Function to print signals */
 
 static void
@@ -2279,7 +2358,7 @@ _print_signal(AtkObject *aobject, FerretSignalType type,
           {
             _speak_caret_event (aobject);
          }
-        else
+        else if (type == FERRET_SIGNAL_TEXT)
           {
             last_caret_offset = atk_text_get_caret_offset (ATK_TEXT (aobject));
           }
@@ -2308,7 +2387,7 @@ _print_signal(AtkObject *aobject, FerretSignalType type,
 /* Update functions */
 
 static void
-_update(TabNumber top_tab, AtkObject *aobject)
+_update (TabNumber top_tab, AtkObject *aobject)
 {
     gint group_num;
 
@@ -2527,3 +2606,19 @@ void _toggle_trackmouse(GtkCheckMenuItem *checkmenuitem,
          }
      }
 }
+
+void _toggle_trackfocus(GtkCheckMenuItem *checkmenuitem,
+  gpointer user_data)
+{
+   if (checkmenuitem->active)
+     {
+       track_focus = TRUE;
+       focus_tracker_id = atk_add_focus_tracker (_print_accessible);
+     }
+   else
+     { 
+       g_print ("No longer tracking focus.\n");
+       track_focus = FALSE;
+       atk_remove_focus_tracker (focus_tracker_id);
+     }
+}
